@@ -45,10 +45,10 @@ class OWElectronBeam(GenericElement):
     electron_beam_beta_v = Setting(0.0)
     electron_beam_alpha_h = Setting(0.0)
     electron_beam_alpha_v = Setting(0.0)
-    electron_beam_eta_h = Setting(0.0)
-    electron_beam_eta_v = Setting(0.0)
-    electron_beam_etap_h = Setting(0.0)
-    electron_beam_etap_v = Setting(0.0)
+    # electron_beam_eta_h = Setting(0.0)
+    # electron_beam_eta_v = Setting(0.0)
+    # electron_beam_etap_h = Setting(0.0)
+    # electron_beam_etap_v = Setting(0.0)
 
     type_of_properties = Setting(1)
     flag_energy_spread = Setting(0)
@@ -111,12 +111,12 @@ class OWElectronBeam(GenericElement):
 
         self.left_box_2_1 = oasysgui.widgetBox(self.electron_beam_box, "", addSpace=False, orientation="vertical", height=150)
 
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xx",   "<x x>   [m^2]",   tooltip="moment_xx",   labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xxp",  "<x x'>  [m.rad]", tooltip="moment_xxp",  labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xpxp", "<x' x'> [rad^2]", tooltip="moment_xpxp", labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_yy",   "<y y>   [m^2]",   tooltip="moment_yy",   labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_yyp",  "<y y'>  [m.rad]", tooltip="moment_yyp",  labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_1, self, "moment_ypyp", "<y' y'> [rad^2]", tooltip="moment_ypyp", labelWidth=260, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xx",   "<x x>   [m^2]",   tooltip="moment_xx",   labelWidth=160, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xxp",  "<x x'>  [m.rad]", tooltip="moment_xxp",  labelWidth=160, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_xpxp", "<x' x'> [rad^2]", tooltip="moment_xpxp", labelWidth=160, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_yy",   "<y y>   [m^2]",   tooltip="moment_yy",   labelWidth=160, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_yyp",  "<y y'>  [m.rad]", tooltip="moment_yyp",  labelWidth=160, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.left_box_2_1, self, "moment_ypyp", "<y' y'> [rad^2]", tooltip="moment_ypyp", labelWidth=160, valueType=float, orientation="horizontal")
 
 
         self.left_box_2_2 = oasysgui.widgetBox(self.electron_beam_box, "", addSpace=False, orientation="vertical", height=150)
@@ -132,15 +132,17 @@ class OWElectronBeam(GenericElement):
         oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_emittance_h", "\u03B5x [m.rad]",tooltip="electron_beam_emittance_h",labelWidth=75, valueType=float, orientation="horizontal")
         oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_alpha_h",     "\u03B1x",        tooltip="electron_beam_alpha_h",    labelWidth=75, valueType=float, orientation="horizontal")
         oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_beta_h",      "\u03B2x [m]",    tooltip="electron_beam_beta_h",     labelWidth=75, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_eta_h",       "\u03B7x",        tooltip="electron_beam_eta_h",      labelWidth=75, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_etap_h",      "\u03B7'x",       tooltip="electron_beam_etap_h",     labelWidth=75, valueType=float, orientation="horizontal")
+        # dispersion effects not considered from Twiss, treated separately
+        # oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_eta_h",       "\u03B7x",        tooltip="electron_beam_eta_h",      labelWidth=75, valueType=float, orientation="horizontal")
+        # oasysgui.lineEdit(self.left_box_2_3_l, self, "electron_beam_etap_h",      "\u03B7'x",       tooltip="electron_beam_etap_h",     labelWidth=75, valueType=float, orientation="horizontal")
 
 
         oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_emittance_v", "\u03B5y [m.rad]",tooltip="electron_beam_emittance_v",labelWidth=75, valueType=float, orientation="horizontal")
         oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_alpha_v",     "\u03B1y",        tooltip="electron_beam_alpha_v",    labelWidth=75,valueType=float, orientation="horizontal")
         oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_beta_v",      "\u03B2y [m]",    tooltip="electron_beam_beta_v",     labelWidth=75, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_eta_v",       "\u03B7y",        tooltip="electron_beam_eta_v",      labelWidth=75, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_etap_v",      "\u03B7'y",       tooltip="electron_beam_etap_v",     labelWidth=75, valueType=float, orientation="horizontal")
+        # dispersion effects not considered from Twiss, treated separately
+        # oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_eta_v",       "\u03B7y",        tooltip="electron_beam_eta_v",      labelWidth=75, valueType=float, orientation="horizontal")
+        # oasysgui.lineEdit(self.left_box_2_3_r, self, "electron_beam_etap_v",      "\u03B7'y",       tooltip="electron_beam_etap_v",     labelWidth=75, valueType=float, orientation="horizontal")
 
         self.set_TypeOfProperties()
 
@@ -170,8 +172,8 @@ class OWElectronBeam(GenericElement):
             congruence.checkPositiveNumber(self.moment_ypyp , "Moment ypyp")
         elif self.type_of_properties == 1:
             congruence.checkPositiveNumber(self.electron_beam_size_h       , "Horizontal Beam Size")
-            congruence.checkPositiveNumber(self.electron_beam_divergence_h , "Vertical Beam Size")
-            congruence.checkPositiveNumber(self.electron_beam_size_v       , "Horizontal Beam Divergence")
+            congruence.checkPositiveNumber(self.electron_beam_divergence_h , "Horizontal Beam Divergence")
+            congruence.checkPositiveNumber(self.electron_beam_size_v       , "Vertical Beam Size")
             congruence.checkPositiveNumber(self.electron_beam_divergence_v , "Vertical Beam Divergence")
         elif self.type_of_properties == 2:
             congruence.checkPositiveNumber(self.electron_beam_emittance_h, "Horizontal Beam Emittance")
@@ -180,10 +182,10 @@ class OWElectronBeam(GenericElement):
             congruence.checkNumber(self.electron_beam_alpha_v, "Vertical Beam Alpha")
             congruence.checkNumber(self.electron_beam_beta_h, "Horizontal Beam Beta")
             congruence.checkNumber(self.electron_beam_beta_v, "Vertical Beam Beta")
-            congruence.checkNumber(self.electron_beam_eta_h, "Horizontal Beam Dispersion Eta")
-            congruence.checkNumber(self.electron_beam_eta_v, "Vertical Beam Dispersion Eta")
-            congruence.checkNumber(self.electron_beam_etap_h, "Horizontal Beam Dispersion Eta'")
-            congruence.checkNumber(self.electron_beam_etap_v, "Vertical Beam Dispersion Eta'")
+            # congruence.checkNumber(self.electron_beam_eta_h, "Horizontal Beam Dispersion Eta")
+            # congruence.checkNumber(self.electron_beam_eta_v, "Vertical Beam Dispersion Eta")
+            # congruence.checkNumber(self.electron_beam_etap_h, "Horizontal Beam Dispersion Eta'")
+            # congruence.checkNumber(self.electron_beam_etap_v, "Vertical Beam Dispersion Eta'")
 
         self.check_magnetic_structure()
 
@@ -191,27 +193,20 @@ class OWElectronBeam(GenericElement):
     def run_shadow4(self):
         raise Exception("To be defined in the superclass")
 
-    def get_electron_beam(self): #to do: remove script
+    def get_electron_beam(self):
         electron_beam = S4ElectronBeam(energy_in_GeV=self.electron_energy_in_GeV,
                                      energy_spread=self.electron_energy_spread,
                                      current=self.ring_current,
                                      # number_of_bunches=self.number_of_bunches,
                                      )
 
-        script = "\n# electron beam"
-        script += "\nfrom shadow4.sources.s4_electron_beam import S4ElectronBeam"
-        script += "\nelectron_beam = S4ElectronBeam(energy_in_GeV=%g,energy_spread=%g,current=%g)" % \
-                  (self.electron_energy_in_GeV,self.electron_energy_spread,self.ring_current)
 
         recalculate_fields = False
 
         if self.type_of_properties == 0:
-            electron_beam.set_moments_horizontal(self.moment_xx,self.moment_xxp,self.moment_xpxp)
+            electron_beam.set_moments_horizontal(self.moment_xx, self.moment_xxp, self.moment_xpxp)
             electron_beam.set_moments_vertical(self.moment_yy, self.moment_yyp, self.moment_ypyp)
 
-            script += "\nelectron_beam.set_moments_horizontal(%g,%g,%g)" % (self.moment_xx,self.moment_xxp,self.moment_xpxp)
-            script +=   "\nelectron_beam.set_moments_vertical(%g,%g,%g)" % (self.moment_yy, self.moment_yyp, self.moment_ypyp)
-
             if recalculate_fields:
 
                 x, xp, y, yp = electron_beam.get_sigmas_all()
@@ -225,25 +220,21 @@ class OWElectronBeam(GenericElement):
                 self.electron_beam_emittance_h = twiss_all[0]
                 self.electron_beam_alpha_h     = twiss_all[1]
                 self.electron_beam_beta_h      = twiss_all[2]
-                self.electron_beam_eta_h       = 0.0
-                self.electron_beam_etap_h      = 0.0
+                # self.electron_beam_eta_h       = 0.0
+                # self.electron_beam_etap_h      = 0.0
                 self.electron_beam_emittance_v = twiss_all[3]
                 self.electron_beam_alpha_v     = twiss_all[4]
                 self.electron_beam_beta_v      = twiss_all[5]
-                self.electron_beam_eta_v       = 0.0
-                self.electron_beam_etap_v      = 0.0
+                # self.electron_beam_eta_v       = 0.0
+                # self.electron_beam_etap_v      = 0.0
 
 
         elif self.type_of_properties == 1:
             electron_beam.set_sigmas_all(sigma_x=self.electron_beam_size_h,
-                                         sigma_y=self.electron_beam_size_v,
                                          sigma_xp=self.electron_beam_divergence_h,
+                                         sigma_y=self.electron_beam_size_v,
                                          sigma_yp=self.electron_beam_divergence_v)
-            script += "\nelectron_beam.set_sigmas_all(sigma_x=%g,sigma_y=%g,sigma_xp=%g,sigma_yp=%g)" % \
-                        (self.electron_beam_size_h,
-                        self.electron_beam_size_v,
-                        self.electron_beam_divergence_h,
-                        self.electron_beam_divergence_v)
+
             if recalculate_fields:
                 moments_all = electron_beam.get_moments_all()
 
@@ -258,39 +249,27 @@ class OWElectronBeam(GenericElement):
                 self.electron_beam_emittance_h = twiss_all[0]
                 self.electron_beam_alpha_h     = twiss_all[1]
                 self.electron_beam_beta_h      = twiss_all[2]
-                self.electron_beam_eta_h       = 0.0
-                self.electron_beam_etap_h      = 0.0
+                # self.electron_beam_eta_h       = 0.0
+                # self.electron_beam_etap_h      = 0.0
                 self.electron_beam_emittance_v = twiss_all[3]
                 self.electron_beam_alpha_v     = twiss_all[4]
                 self.electron_beam_beta_v      = twiss_all[5]
-                self.electron_beam_eta_v       = 0.0
-                self.electron_beam_etap_v      = 0.0
+                # self.electron_beam_eta_v       = 0.0
+                # self.electron_beam_etap_v      = 0.0
 
         elif self.type_of_properties == 2:
             electron_beam.set_twiss_horizontal(self.electron_beam_emittance_h,
                                              self.electron_beam_alpha_h,
                                              self.electron_beam_beta_h,
-                                             self.electron_beam_eta_h,
-                                             self.electron_beam_etap_h)
+                                             0.0,
+                                             0.0,
+                                               )
             electron_beam.set_twiss_vertical(self.electron_beam_emittance_v,
                                              self.electron_beam_alpha_v,
                                              self.electron_beam_beta_v,
-                                             self.electron_beam_eta_v,
-                                             self.electron_beam_etap_v)
-            script += "\nelectron_beam.set_twiss_horizontal(%g,%g,%g,%g,%g)" % (
-                  self.electron_beam_emittance_h,
-                  self.electron_beam_alpha_h,
-                  self.electron_beam_beta_h,
-                  self.electron_beam_eta_h,
-                  self.electron_beam_etap_h,
-            )
-            script += "\nelectron_beam.set_twiss_vertical(%g,%g,%g,%g,%g)" % (
-                  self.electron_beam_emittance_v,
-                  self.electron_beam_alpha_v,
-                  self.electron_beam_beta_v,
-                  self.electron_beam_eta_v,
-                  self.electron_beam_etap_v,
-            )
+                                             0.0,
+                                             0.0,
+                                             )
 
 
 
@@ -312,7 +291,6 @@ class OWElectronBeam(GenericElement):
                 self.moment_ypyp = moments_all[5]
         elif self.type_of_properties == 3:
             electron_beam.set_moments_all(0,0,0,0,0,0)
-            script += "\nelectron_beam.set_moments_all(0,0,0,0,0,0)"
 
         return electron_beam
 
@@ -336,5 +314,17 @@ class OWElectronBeam(GenericElement):
         self.electron_beam_divergence_h = xp
         self.electron_beam_divergence_v = yp
 
+        twiss_all = electron_beam.get_twiss_no_dispersion_all()
+        self.electron_beam_emittance_h = twiss_all[0]
+        self.electron_beam_alpha_h = twiss_all[1]
+        self.electron_beam_beta_h = twiss_all[2]
+        # self.electron_beam_eta_h = 0.0
+        # self.electron_beam_etap_h = 0.0
+        self.electron_beam_emittance_v = twiss_all[3]
+        self.electron_beam_alpha_v = twiss_all[4]
+        self.electron_beam_beta_v = twiss_all[5]
+        # self.electron_beam_eta_v = 0.0
+        # self.electron_beam_etap_v = 0.0
+
         self.type_of_properties = 1
         self.set_TypeOfProperties()
