@@ -1,5 +1,5 @@
 import numpy
-import os
+import os, copy
 from scipy.optimize import root
 
 from PyQt5.QtWidgets import QDialog, QGridLayout, QWidget, QDialogButtonBox, QFileDialog
@@ -10,6 +10,8 @@ from matplotlib.figure import Figure
 try:    from mpl_toolkits.mplot3d import Axes3D  # plot 3D
 except: pass
 
+from silx.gui.plot import Plot2D
+
 from orangecanvas.resources import icon_loader
 from orangecanvas.scheme.node import SchemeNode
 
@@ -22,19 +24,23 @@ from oasys.widgets import congruence
 
 from oasys.util.oasys_util import read_surface_file, write_surface_file
 from oasys.util.oasys_objects import OasysPreProcessorData
+from oasys.util.oasys_objects import OasysSurfaceData
 
 from syned.beamline.shape import Rectangle
 from syned.beamline.shape import Ellipse
 
 from srxraylib.metrology import profiles_simulation
 
-from orangecontrib.shadow4.widgets.gui.ow_optical_element import OWOpticalElement, optical_element_inputs, SUBTAB_INNER_BOX_WIDTH
+from orangecontrib.shadow4.widgets.gui.ow_optical_element import OWOpticalElement, SUBTAB_INNER_BOX_WIDTH
 
 from shadow4.beamline.s4_beamline_element_movements import S4BeamlineElementMovements
+from orangecontrib.shadow4.widgets.gui.ow_optical_element_with_surface_shape_render import ShowSurfaceShapeDialog
 
 class OWOpticalElementWithSurfaceShape(OWOpticalElement):
-    inputs = optical_element_inputs()
-    inputs.append(("PreProcessor Data", OasysPreProcessorData, "set_surface_data"))
+    inputs = copy.deepcopy(OWOpticalElement.inputs)
+    inputs.append(("Surface Data", OasysSurfaceData, "set_oasys_surface_data"))
+    inputs.append(("PreProcessor_Data", OasysPreProcessorData, "set_oasys_preprocessor_data"))
+
 
     #########################################################
     # surface shape
@@ -322,14 +328,14 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
         self.focusing_external_toroid = oasysgui.widgetBox(self.focusing_box, "", addSpace=False, orientation="vertical",
                                                          height=150)
         self.le_torus_major_radius = oasysgui.lineEdit(self.focusing_external_toroid, self, "torus_major_radius",
-                                                       "Torus Major Radius [m] (Rtan-Rsag)", labelWidth=260, valueType=float,
+                                                       "Toroid Major Radius [m] (optical, Rtan)", labelWidth=260, valueType=float,
                                                        orientation="horizontal", tooltip="torus_major_radius")
         self.le_torus_minor_radius = oasysgui.lineEdit(self.focusing_external_toroid, self, "torus_minor_radius",
-                                                       "Torus Minor Radius [m] (Rsag)", labelWidth=260, valueType=float,
+                                                       "Toroid Minor Radius [m] (Rsag)", labelWidth=260, valueType=float,
                                                        orientation="horizontal", tooltip="torus_minor_radius")
 
 
-        gui.comboBox(self.focusing_external_toroid, self, "toroidal_mirror_pole_location", label="Torus pole location",
+        gui.comboBox(self.focusing_external_toroid, self, "toroidal_mirror_pole_location", label="Toroid pole location",
                      labelWidth=145,
                      items=["lower/outer (concave/concave)",
                             "lower/inner (concave/convex)",
@@ -420,11 +426,14 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
         self.mod_surf_err_box_1 = oasysgui.widgetBox(box, "", addSpace=False, orientation="horizontal")
 
         self.le_ms_defect_file_name = oasysgui.lineEdit(self.mod_surf_err_box_1, self, "ms_defect_file_name",
-                                                        "File name", labelWidth=60, valueType=str,
-                                                        orientation="horizontal")
+                                                        "File", tooltip="ms_defect_file_name", labelWidth=40,
+                                                        valueType=str, orientation="horizontal")
 
         gui.button(self.mod_surf_err_box_1, self, "...", callback=self.select_defect_file_name, width=30)
-        gui.button(self.mod_surf_err_box_1, self, "View", callback=self.view_surface_error_data_file, width=40)
+        gui.button(self.mod_surf_err_box_1, self, "View Surf", callback=self.view_surface_error_data_file, width=65,
+                   tooltip="Render data in surface mode [slow]")
+        gui.button(self.mod_surf_err_box_1, self, "View Img", callback=self.view_image_error_data_file, width=65,
+                   tooltip="Render data in image mode [slow]")
 
         self.modified_surface_tab_visibility()
 
@@ -578,7 +587,7 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
 
         if not is_init: self.__change_icon_from_surface_type()
 
-    def set_surface_data(self, oasys_data : OasysPreProcessorData):
+    def set_oasys_preprocessor_data(self, oasys_data : OasysPreProcessorData):
         if not oasys_data is None:
             if not oasys_data.error_profile_data is None:
                 try:
@@ -595,24 +604,30 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
                 except Exception as exception:
                     self.prompt_exception(exception)
 
+    def set_oasys_surface_data(self, oasys_data : OasysSurfaceData):
+        if oasys_data is not None:
+            if oasys_data.surface_data_file is not None:
+                try:
+                    self.ms_defect_file_name = oasys_data.surface_data_file
+                    self.modified_surface = 1
+                    self.modified_surface_tab_visibility()
+                    self.congruence_surface_data_file(oasys_data.xx, oasys_data.yy, oasys_data.zz)
+                except Exception as exception:
+                    self.prompt_exception(exception)
+
     def congruence_surface_data_file(self, xx=None, yy=None, zz=None):
         # check congruence of limits and ask for corrections
         surface_data_file = self.ms_defect_file_name
 
         if not os.path.isfile(surface_data_file): raise Exception("File %s not found." % surface_data_file)
 
-        ask_for_fix = False
-        if self.is_infinite == 1:
-            ask_for_fix = True
-        else:
-            if self.oe_shape != 0:
-                ask_for_fix = True
+        ask_for_fix = self.is_infinite == 1 or self.oe_shape != 0
 
         if ask_for_fix:
             if xx is None or yy is None or zz is None: xx, yy, zz = read_surface_file(surface_data_file)
 
-            print(">>>> File limits: ", xx.min(), xx.max(), yy.min(), yy.max())
-            print(">>>> Current limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
+            print(">> File limits: ", xx.min(), xx.max(), yy.min(), yy.max())
+            print(">> Current limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
 
             if (xx.min() > -self.dim_x_minus) or \
                     (xx.max() > self.dim_x_plus) or \
@@ -629,22 +644,29 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
                     self.dim_y_minus = numpy.min((-yy.min(), self.dim_y_minus))
                     self.dim_y_plus = numpy.min((yy.max(), self.dim_y_plus))
 
-                    print(">>>> NEW limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
+                    print(">> NEW limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
 
                     self.dimensions_tab_visibility()
                 else:
-                    print(">>>> **NOT CHANGED** limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
+                    print(">> **NOT CHANGED** limits: ", self.dim_x_minus, self.dim_x_plus, self.dim_y_minus, self.dim_x_plus)
 
     def view_surface_error_data_file(self):
         try:
-            dialog = self.ShowSurfaceErrorDataFileDialog(parent=self)
+            dialog = ShowSurfaceErrorDataFileDialog(parent=self, file_name=self.ms_defect_file_name)
+            dialog.show()
+        except Exception as exception:
+            self.prompt_exception(exception)
+
+    def view_image_error_data_file(self):
+        try:
+            dialog = ShowImageErrorDataFileDialog(parent=self, file_name=self.ms_defect_file_name)
             dialog.show()
         except Exception as exception:
             self.prompt_exception(exception)
 
     def view_surface_shape_data(self):
         try:
-            dialog = self.ShowSurfaceShapeDialog(parent=self)
+            dialog = ShowSurfaceShapeDialog(parent=self)
             dialog.show()
         except Exception as exception:
             self.prompt_exception(exception)
@@ -709,7 +731,7 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
         else:                                      return self.image_side_focal_distance
 
     def get_boundary_shape(self):
-        if self.is_infinite: return None
+        if self.is_infinite == 1: return None
         else:
             if self.oe_shape == 0: # Rectangular
                 return Rectangle(x_left=-self.dim_x_minus, x_right=self.dim_x_plus,
@@ -719,291 +741,111 @@ class OWOpticalElementWithSurfaceShape(OWOpticalElement):
                                b_axis_min=-self.dim_y_minus, b_axis_max=self.dim_y_plus)
 
 
-    class ShowSurfaceErrorDataFileDialog(QDialog):
-        def __init__(self, parent=None):
-            QDialog.__init__(self, parent)
-            self.setWindowTitle('Surface Error Profile')
-            self.setFixedHeight(700)
-            layout = QGridLayout(self)
+class ShowSurfaceErrorDataFileDialog(QDialog):
+    def __init__(self, parent=None, file_name=None):
+        QDialog.__init__(self, parent)
+        self.setWindowTitle('Surface Error Profile')
+        self.setFixedHeight(700)
+        layout = QGridLayout(self)
 
-            figure = Figure(figsize=(8, 7))
-            figure.patch.set_facecolor('white')
+        figure = Figure(figsize=(8, 7))
+        figure.patch.set_facecolor('white')
 
-            axis = figure.add_subplot(111, projection='3d')
-            axis.set_xlabel("X [m]")
-            axis.set_ylabel("Y [m]")
-            axis.set_zlabel("Z [nm]")
+        axis = figure.add_subplot(111, projection='3d')
+        axis.set_xlabel("X [m]")
+        axis.set_ylabel("Y [m]")
+        axis.set_zlabel("Z [nm]")
 
-            figure_canvas = FigureCanvas3D(ax=axis, fig=figure, show_legend=False, show_buttons=False)
-            figure_canvas.setFixedWidth(500)
-            figure_canvas.setFixedHeight(645)
+        figure_canvas = FigureCanvas3D(ax=axis, fig=figure, show_legend=False, show_buttons=False)
+        figure_canvas.setFixedWidth(500)
+        figure_canvas.setFixedHeight(645)
 
-            xx, yy, zz = read_surface_file(parent.ms_defect_file_name)
+        xx, yy, zz = read_surface_file(file_name) # parent.ms_defect_file_name)
 
-            x_to_plot, y_to_plot = numpy.meshgrid(xx, yy)
-            zz_slopes = zz.T
+        x_to_plot, y_to_plot = numpy.meshgrid(xx, yy)
+        zz_slopes = zz.T
 
-            axis.plot_surface(x_to_plot, y_to_plot,  zz*1e9, rstride=1, cstride=1, cmap=cm.autumn, linewidth=0.5, antialiased=True)
+        axis.plot_surface(x_to_plot, y_to_plot,  zz*1e9, rstride=1, cstride=1, cmap=cm.autumn, linewidth=0.5, antialiased=True)
 
-            sloperms = profiles_simulation.slopes(zz_slopes, xx, yy, return_only_rms=1)
+        sloperms = profiles_simulation.slopes(zz_slopes, xx, yy, return_only_rms=1)
 
-            title = ' Slope error rms in X direction: %f $\mu$rad' % (sloperms[0]*1e6) + '\n' + \
-                    ' Slope error rms in Y direction: %f $\mu$rad' % (sloperms[1]*1e6) + '\n' + \
-                    ' Figure error rms in X direction: %f nm' % (round(zz_slopes[:, 0].std()*1e9, 6)) + '\n' + \
-                    ' Figure error rms in Y direction: %f nm' % (round(zz_slopes[0, :].std()*1e9, 6))
+        title = ' Slope error rms in X: %g $\mu$rad' % (sloperms[0]*1e6) +  \
+                ' in Y: %g $\mu$rad' % (sloperms[1]*1e6) + '\n' + \
+                ' Figure error rms: %g nm' % (zz.std()*1e9)
 
-            axis.set_title(title)
-            figure_canvas.draw()
-            axis.mouse_init()
+        axis.set_title(title)
+        figure_canvas.draw()
+        axis.mouse_init()
 
-            widget = QWidget(parent=self)
-            container = oasysgui.widgetBox(widget, "", addSpace=False, orientation="horizontal", width=500)
-            #gui.button(container, self, "Export Surface (.hdf5)", callback=self.save_oasys_surface)
-            gui.button(container, self, "Close", callback=self.accept)
+        widget = QWidget(parent=self)
+        container = oasysgui.widgetBox(widget, "", addSpace=False, orientation="horizontal", width=500)
+        #gui.button(container, self, "Export Surface (.hdf5)", callback=self.save_oasys_surface)
+        gui.button(container, self, "Close", callback=self.accept)
 
-            layout.addWidget(figure_canvas, 0, 0)
-            layout.addWidget(widget, 1, 0)
+        layout.addWidget(figure_canvas, 0, 0)
+        layout.addWidget(widget, 1, 0)
 
-            self.setLayout(layout)
+        self.setLayout(layout)
 
-    class ShowSurfaceShapeDialog(QDialog):
-        c1  = 0.0
-        c2  = 0.0
-        c3  = 0.0
-        c4  = 0.0
-        c5  = 0.0
-        c6  = 0.0
-        c7  = 0.0
-        c8  = 0.0
-        c9  = 0.0
-        c10 = 0.0
+class ShowImageErrorDataFileDialog(QDialog):
+    def __init__(self, parent=None, file_name=None):
+        QDialog.__init__(self, parent)
+        self.setWindowTitle('Surface Error Profile')
+        # self.setFixedHeight(700)
+        layout = QGridLayout(self)
 
-        torus_major_radius = 0.0
-        torus_minor_radius = 0.0
+        dataX, dataY, data2D = read_surface_file(file_name) # parent.ms_defect_file_name)
 
-        xx = None
-        yy = None
-        zz = None
+        ##################
+        zz_slopes = data2D.T
 
-        bin_x = 100
-        bin_y = 1000
+        sloperms = profiles_simulation.slopes(zz_slopes, dataX, dataY, return_only_rms=1)
 
-        def __init__(self, parent=None):
-            QDialog.__init__(self, parent)
-            self.setWindowTitle('O.E. Surface Shape')
-            self.setFixedWidth(750)
+        title = ' Slope error rms in X: %g $\mu$rad' % (sloperms[0]*1e6) +  \
+                ' in Y: %g $\mu$rad' % (sloperms[1]*1e6) + '\n' + \
+                ' Figure error rms: %g nm' % (data2D.std()*1e9)
+        ##################
 
-            layout = QGridLayout(self)
+        origin = (dataX[0], dataY[0])
+        scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])
 
-            figure = Figure(figsize=(100, 100))
-            figure.patch.set_facecolor('white')
+        colormap = {"name": "temperature", "normalization": "linear",
+                    "autoscale": True, "vmin": 0, "vmax": 0, "colors": 256}
 
-            axis = figure.add_subplot(111, projection='3d')
-            axis.set_xlabel("X [" + parent.workspace_units_label + "]")
-            axis.set_ylabel("Y [" + parent.workspace_units_label + "]")
-            axis.set_zlabel("Z [" + parent.workspace_units_label + "]")
 
-            figure_canvas = FigureCanvas3D(ax=axis, fig=figure, show_legend=False, show_buttons=False)
-            figure_canvas.setFixedWidth(500)
-            figure_canvas.setFixedHeight(500)
+        tmp = Plot2D()
+        tmp.resetZoom()
+        tmp.setXAxisAutoScale(True)
+        tmp.setYAxisAutoScale(True)
+        tmp.setGraphGrid(False)
+        tmp.setKeepDataAspectRatio(True)
+        tmp.yAxisInvertedAction.setVisible(False)
+        tmp.setXAxisLogarithmic(False)
+        tmp.setYAxisLogarithmic(False)
+        tmp.getMaskAction().setVisible(False)
+        tmp.getRoiAction().setVisible(False)
+        tmp.getColormapAction().setVisible(True)
+        tmp.setKeepDataAspectRatio(False)
+        tmp.addImage(data2D, legend="1", scale=scale, origin=origin, colormap=colormap, replace=True)
+        tmp.setActiveImage("1")
+        tmp.setGraphXLabel("X [m] (%d pixels)" % dataX.size)
+        tmp.setGraphYLabel("Y [m] (%d pixels)" % dataY.size)
+        tmp.setGraphTitle("Z [m]\n" + title)
 
-            X, Y, z_values = self.calculate_surface(parent, 100, 100)
+        widget = QWidget(parent=self)
 
-            axis.plot_surface(X, Y, z_values, rstride=1, cstride=1, cmap=cm.autumn, linewidth=0.5, antialiased=True)
+        layout.addWidget(tmp, 0, 0)
+        layout.addWidget(widget, 1, 0)
 
-            if parent.surface_shape_type == 5:
-                axis.set_title("Surface from Torus equation:\n" +
-                               "[(Z + R + r)" + u"\u00B2" +
-                               " + Y" + u"\u00B2" +
-                               " + X" + u"\u00B2" +
-                               " + R" + u"\u00B2" +
-                               " - r" + u"\u00B2"
-                               + "]" + u"\u00B2" +
-                               "= 4R" + u"\u00B2" + "[(Z + R + r)" + u"\u00B2" + " + Y" + u"\u00B2" + "]")
-            else:
-                title_head = "Surface from generated conic coefficients:\n"
-                title = ""
-                max_dim = 40
+        self.setLayout(layout)
 
-                if self.c1 != 0: title +=       str(self.c1) + u"\u00B7" + "X" + u"\u00B2"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c2 < 0 or (self.c2 > 0 and title == ""): title +=       str(self.c2) + u"\u00B7" + "Y" + u"\u00B2"
-                elif self.c2 > 0                                 : title += "+" + str(self.c2) + u"\u00B7" + "Y" + u"\u00B2"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c3 < 0 or (self.c3 > 0 and title == ""): title +=       str(self.c3) + u"\u00B7" + "Z" + u"\u00B2"
-                elif self.c3 > 0                                 : title += "+" + str(self.c3) + u"\u00B7" + "Z" + u"\u00B2"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c4 < 0 or (self.c4 > 0 and title == ""): title +=       str(self.c4) + u"\u00B7" + "XY"
-                elif self.c4 > 0                                 : title += "+" + str(self.c4) + u"\u00B7" + "XY"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c5 < 0 or (self.c5 > 0 and title == ""): title +=       str(self.c5) + u"\u00B7" + "YZ"
-                elif self.c5 > 0                                 : title += "+" + str(self.c5) + u"\u00B7" + "YZ"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c6 < 0 or (self.c6 > 0 and title == ""): title +=       str(self.c6) + u"\u00B7" + "XZ"
-                elif self.c6 > 0                                 : title += "+" + str(self.c6) + u"\u00B7" + "XZ"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c7 < 0 or (self.c7 > 0 and title == ""): title +=       str(self.c7) + u"\u00B7" + "X"
-                elif self.c7 > 0                                 : title += "+" + str(self.c7) + u"\u00B7" + "X"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c8 < 0 or (self.c8 > 0 and title == ""): title +=       str(self.c8) + u"\u00B7" + "Y"
-                elif self.c8 > 0                                 : title += "+" + str(self.c8) + u"\u00B7" + "Y"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c9 < 0 or (self.c9 > 0 and title == ""): title +=       str(self.c9) + u"\u00B7" + "Z"
-                elif self.c9 > 0                                 : title += "+" + str(self.c9) + u"\u00B7" + "Z"
-                if len(title) >=  max_dim:
-                    title_head += title + "\n"
-                    title = ""
-                if self.c10< 0 or (self.c10> 0 and title == ""): title +=       str(self.c10)
-                elif self.c10> 0                                 : title += "+" + str(self.c10)
-
-                axis.set_title(title_head + title + " = 0")
-
-            figure_canvas.draw()
-            axis.mouse_init()
-
-            widget    = QWidget(parent=self)
-            container = oasysgui.widgetBox(widget, "", addSpace=False, orientation="vertical", width=220)
-
-            if parent.surface_shape_type == 5:
-                surface_box = oasysgui.widgetBox(container, "Torus Parameters", addSpace=False, orientation="vertical", width=220, height=375)
-
-                le_torus_major_radius = oasysgui.lineEdit(surface_box, self, "torus_major_radius" , "R" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_torus_minor_radius = oasysgui.lineEdit(surface_box, self, "torus_minor_radius" , "r" , labelWidth=60, valueType=float, orientation="horizontal")
-
-                le_torus_major_radius.setReadOnly(True)
-                le_torus_minor_radius.setReadOnly(True)
-            else:
-                surface_box = oasysgui.widgetBox(container, "Conic Coefficients", addSpace=False, orientation="vertical", width=220, height=375)
-
-                label  = "c[1]" + u"\u00B7" + "X" + u"\u00B2" + " + c[2]" + u"\u00B7" + "Y" + u"\u00B2" + " + c[3]" + u"\u00B7" + "Z" + u"\u00B2" + " +\n"
-                label += "c[4]" + u"\u00B7" + "XY" + " + c[5]" + u"\u00B7" + "YZ" + " + c[6]" + u"\u00B7" + "XZ" + " +\n"
-                label += "c[7]" + u"\u00B7" + "X" + " + c[8]" + u"\u00B7" + "Y" + " + c[9]" + u"\u00B7" + "Z" + " + c[10] = 0"
-
-                gui.label(surface_box, self, label)
-
-                gui.separator(surface_box, 10)
-
-                le_0 = oasysgui.lineEdit(surface_box, self, "c1" , "c[1]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_1 = oasysgui.lineEdit(surface_box, self, "c2" , "c[2]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_2 = oasysgui.lineEdit(surface_box, self, "c3" , "c[3]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_3 = oasysgui.lineEdit(surface_box, self, "c4" , "c[4]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_4 = oasysgui.lineEdit(surface_box, self, "c5" , "c[5]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_5 = oasysgui.lineEdit(surface_box, self, "c6" , "c[6]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_6 = oasysgui.lineEdit(surface_box, self, "c7" , "c[7]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_7 = oasysgui.lineEdit(surface_box, self, "c8" , "c[8]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_8 = oasysgui.lineEdit(surface_box, self, "c9" , "c[9]" , labelWidth=60, valueType=float, orientation="horizontal")
-                le_9 = oasysgui.lineEdit(surface_box, self, "c10", "c[10]", labelWidth=60, valueType=float, orientation="horizontal")
-
-                le_0.setReadOnly(True)
-                le_1.setReadOnly(True)
-                le_2.setReadOnly(True)
-                le_3.setReadOnly(True)
-                le_4.setReadOnly(True)
-                le_5.setReadOnly(True)
-                le_6.setReadOnly(True)
-                le_7.setReadOnly(True)
-                le_8.setReadOnly(True)
-                le_9.setReadOnly(True)
-
-            export_box = oasysgui.widgetBox(container, "Export", addSpace=False, orientation="vertical", width=220)
-
-            bin_box = oasysgui.widgetBox(export_box, "", addSpace=False, orientation="horizontal")
-
-            oasysgui.lineEdit(bin_box, self, "bin_x" , "Bins X" , labelWidth=40, valueType=float, orientation="horizontal")
-            oasysgui.lineEdit(bin_box, self, "bin_y" , " x Y" , labelWidth=30, valueType=float, orientation="horizontal")
-
-            gui.button(export_box, self, "Export Surface (.hdf5)", callback=self.save_oasys_surface)
-
-            bbox = QDialogButtonBox(QDialogButtonBox.Ok)
-            bbox.accepted.connect(self.accept)
-            layout.addWidget(figure_canvas, 0, 0)
-            layout.addWidget(widget, 0, 1)
-            layout.addWidget(bbox, 1, 0, 1, 2)
-
-            self.setLayout(layout)
-
-        def calculate_surface(self, parent, bin_x=100, bin_y=100, sign=-1):
-            if parent.is_infinite == 0:
-                x_min = -10
-                x_max = 10
-                y_min = -10
-                y_max = 10
-            else:
-                x_min = -parent.dim_x_minus
-                x_max = parent.dim_x_plus
-                y_min = -parent.dim_y_minus
-                y_max = parent.dim_y_plus
-
-            self.xx = numpy.linspace(x_min, x_max, bin_x + 1)
-            self.yy = numpy.linspace(y_min, y_max, bin_y + 1)
-
-            X, Y = numpy.meshgrid(self.xx, self.yy)
-
-            if parent.surface_shape_type == 5:
-                self.torus_major_radius = parent.torus_major_radius
-                self.torus_minor_radius = parent.torus_minor_radius
-
-                sign = -1 if parent.toroidal_mirror_pole_location <= 1 else 1
-
-                z_values = sign*(numpy.sqrt((self.torus_major_radius
-                                             + numpy.sqrt(self.torus_minor_radius**2-X**2))**2
-                                            - Y**2)
-                                - self.torus_major_radius - self.torus_minor_radius)
-                z_values[numpy.where(numpy.isnan(z_values))] = 0.0
-            else:
-                self.c1 = round(parent.conic_coefficient_0, 10)
-                self.c2 = round(parent.conic_coefficient_1, 10)
-                self.c3 = round(parent.conic_coefficient_2, 10)
-                self.c4 = round(parent.conic_coefficient_3, 10)
-                self.c5 = round(parent.conic_coefficient_4, 10)
-                self.c6 = round(parent.conic_coefficient_5, 10)
-                self.c7 = round(parent.conic_coefficient_6, 10)
-                self.c8 = round(parent.conic_coefficient_7, 10)
-                self.c9 = round(parent.conic_coefficient_8, 10)
-                self.c10= round(parent.conic_coefficient_9, 10)
-
-                def equation_to_solve(Z):
-                    return self.c1*(X**2) + self.c2*(Y**2) + self.c3*(Z**2) + self.c4*X*Y + self.c5*Y*Z + self.c6*X*Z + self.c7*X + self.c8*Y + self.c9*Z + self.c10
-
-                z_start = numpy.zeros(X.shape)
-                result = root(equation_to_solve, z_start, method='df-sane', tol=None)
-
-                z_values = result.x if result.success else z_start
-
-            self.zz = z_values
-
-            return X, Y, z_values
-
-        def check_values(self):
-            congruence.checkStrictlyPositiveNumber(self.bin_x, "Bins X")
-            congruence.checkStrictlyPositiveNumber(self.bin_y, "Bins Y")
-
-        def save_oasys_surface(self):
-            try:
-                file_path = QFileDialog.getSaveFileName(self, "Save Surface in Oasys (hdf5) Format", ".", "HDF5 format (*.hdf5)")[0]
-
-                if not file_path is None and not file_path.strip() == "":
-                    self.check_values()
-
-                    self.calculate_surface(self.parent(), int(self.bin_x), int(self.bin_y))
-
-                    write_surface_file(self.zz, numpy.round(self.xx, 8), numpy.round(self.yy, 8), file_path)
-            except Exception as exception:
-                self.parent().prompt_exception(exception)
 
+if __name__ == "__main__":
+    import sys
+    from PyQt5.QtWidgets import QApplication
+    app = QApplication(sys.argv)
+    dialog1 = ShowImageErrorDataFileDialog(file_name="/home/srio/Oasys/conic.h5")
+    dialog2 = ShowSurfaceErrorDataFileDialog(file_name="/home/srio/Oasys/conic.h5")
+    dialog1.show()
+    dialog2.show()
+    app.exec()
\ No newline at end of file
