@@ -8,6 +8,6 @@ DESCRIPTION = "Optics."
 
 BACKGROUND = "#b9d47a"
 
-ICON = "icons/opticalElements.png"
+ICON = "icons/optical_elements.png"
 
-PRIORITY = 1005
\ No newline at end of file
+PRIORITY = 1005
