@@ -27,18 +27,18 @@ from orangecontrib.shadow4.widgets.gui.ow_optical_element_with_surface_shape imp
 from orangecontrib.shadow4.util.shadow4_objects import BraggPreProcessorData
 import copy
 
-class OWCrystal(OWOpticalElementWithSurfaceShape):
-    name = "Generic Crystal"
-    description = "Shadow Crystal"
-    icon = "icons/plane_crystal.png"
-
-    priority = 1.3
-
-    def get_oe_type(self):
-        return "crystal", "Crystal"
-
-    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
-    inputs.append(("Bragg PreProcessor Data", BraggPreProcessorData, "setBraggProcessorData"))
+class _OWCrystal(OWOpticalElementWithSurfaceShape):
+    # name = "Generic Crystal"
+    # description = "Shadow Crystal"
+    # icon = "icons/plane_crystal.png"
+    #
+    # priority = 1.3
+    #
+    # def get_oe_type(self):
+    #     return "crystal", "Crystal"
+    #
+    # inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
+    # inputs.append(("Bragg PreProcessor Data", BraggPreProcessorData, "setBraggProcessorData"))
 
     #########################################################
     # crystal
@@ -77,9 +77,10 @@ class OWCrystal(OWOpticalElementWithSurfaceShape):
     asymmetric_cut = Setting(0)
     planes_angle = Setting(0.0)
     below_onto_bragg_planes = Setting(-1)
+    method_efields_management = Setting(0)
 
     def __init__(self):
-        super(OWCrystal, self).__init__()
+        super(_OWCrystal, self).__init__()
         # with crystals no "internal surface parameters" allowed. Fix value and hide selecting combo:
         self.surface_shape_parameters = 1
         self.surface_shape_internal_external_box.setVisible(False)
@@ -211,6 +212,17 @@ class OWCrystal(OWOpticalElementWithSurfaceShape):
                           valueType=float, orientation="horizontal")
 
 
+        #
+        advanced_box = oasysgui.widgetBox(subtab_crystal_diffraction, "Advanced Settings", addSpace=True, orientation="vertical")
+
+        gui.comboBox(advanced_box, self, "method_efields_management", tooltip="method_efields_management",
+                     label="manage electric fields", labelWidth=160,
+                     items=["via Jones calculus (S4)",
+                            "via rotations (S3)"],
+                     sendSelectedValue=False, orientation="horizontal",
+                     callback=self.crystal_diffraction_tab_visibility)
+
+
         self.crystal_diffraction_tab_visibility()
 
     def populate_tab_crystal_geometry(self, subtab_crystal_geometry):
@@ -265,7 +277,7 @@ class OWCrystal(OWOpticalElementWithSurfaceShape):
                                                              orientation="vertical")
 
         self.le_thickness_1 = oasysgui.lineEdit(self.thickness_box_1, self,
-                                                "thickness", "Crystal thickness [m]",tooltip="thickness",
+                                                "thickness", "Crystal thickness [m]", tooltip="thickness",
                                                 valueType=float, labelWidth=260, orientation="horizontal")
 
         # self.set_BraggLaue()
@@ -439,6 +451,7 @@ class OWCrystal(OWOpticalElementWithSurfaceShape):
                 f_bragg_a=True if self.asymmetric_cut else False,
                 f_ext=0,
                 material_constants_library_flag=self.diffraction_calculation,
+                method_efields_management=self.method_efields_management,
             )
 
         elif self.surface_shape_type == 1:
@@ -637,6 +650,26 @@ class OWCrystal(OWOpticalElementWithSurfaceShape):
             elif self.surface_shape_type == 5: return S4ToroidCrystalElement()
             elif self.surface_shape_type == 6: return S4ConicCrystalElement()
 
+    def _post_trace_operations(self, output_beam, footprint, element, beamline):
+        angle_radial, angle_radial_out, _ = element.get_coordinates().get_angles()
+
+        self.incidence_angle_deg   = numpy.round(numpy.degrees(angle_radial),5)
+        self.reflection_angle_deg  = numpy.round(numpy.degrees(angle_radial_out),5)
+        self.incidence_angle_mrad  = numpy.round(1e3 * (numpy.pi / 2 - angle_radial),5)
+        self.reflection_angle_mrad = numpy.round(1e3 * (numpy.pi / 2 - angle_radial_out),5)
+
+class OWCrystal(_OWCrystal):
+    name = "Generic Crystal"
+    description = "Shadow Crystal"
+    icon = "icons/plane_crystal.png"
+
+    priority = 1.3
+
+    def get_oe_type(self):
+        return "crystal", "Crystal"
+
+    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
+    inputs.append(("Bragg PreProcessor Data", BraggPreProcessorData, "setBraggProcessorData"))
 
 if __name__ == "__main__":
     from shadow4.beamline.s4_beamline import S4Beamline
