@@ -27,18 +27,19 @@ from orangecontrib.shadow4.widgets.gui.ow_optical_element_with_surface_shape imp
 from orangecontrib.shadow4.util.shadow4_objects import VlsPgmPreProcessorData
 import copy
 
-class OWGrating(OWOpticalElementWithSurfaceShape):
-    name = "Generic Grating"
-    description = "Shadow Grating"
-    icon = "icons/plane_grating.png"
 
-    priority = 1.390
-
-    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
-    inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "setVlsPgmPreProcessorData"))
-
-    def get_oe_type(self):
-        return "grating", "Grating"
+class _OWGrating(OWOpticalElementWithSurfaceShape):
+    # name = "Generic Grating"
+    # description = "Shadow Grating"
+    # icon = "icons/plane_grating.png"
+    #
+    # priority = 1.390
+    #
+    # inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
+    # inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "setVlsPgmPreProcessorData"))
+    #
+    # def get_oe_type(self):
+    #     return "grating", "Grating"
 
     #########################################################
     # grating
@@ -54,7 +55,7 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
     # file_refl = Setting("")
 
     def __init__(self):
-        super(OWGrating, self).__init__()
+        super(_OWGrating, self).__init__()
         # with gratings no "internal surface parameters" allowed. Fix value and hide selecting combo:
         self.surface_shape_parameters = 1
         self.surface_shape_internal_external_box.setVisible(False)
@@ -296,8 +297,8 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
             self.surface_shape_type = 0
             self.surface_shape_tab_visibility()
 
-            self.source_plane_distance = data.d_mirror_to_grating/2 * 1e-3
-            self.image_plane_distance = data.d_grating_to_exit_slits * 1e-3
+            self.source_plane_distance = data.d_mirror_to_grating/2
+            self.image_plane_distance = data.d_grating_to_exit_slits
 
             self.angles_respect_to = 0
             self.incidence_angle_deg = data.alpha
@@ -309,10 +310,10 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
             self.order = -1
 
             self.f_ruling = 1
-            self.ruling = data.shadow_coeff_0 * 1e3
-            self.ruling_coeff_linear = data.shadow_coeff_1 * 1e6
-            self.ruling_coeff_quadratic = data.shadow_coeff_2 * 1e9
-            self.ruling_coeff_cubic = data.shadow_coeff_3 * 1e12
+            self.ruling = data.shadow_coeff_0
+            self.ruling_coeff_linear = data.shadow_coeff_1
+            self.ruling_coeff_quadratic = data.shadow_coeff_2
+            self.ruling_coeff_cubic = data.shadow_coeff_3
             self.ruling_coeff_quartic = 0.0
             self.grating_diffraction_tab_visibility()
 
@@ -410,7 +411,7 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
                 coating=None,
                 coating_thickness=None,
                 order=self.order,
-                f_ruling=0 if (self.f_ruling==0) else 5,
+                f_ruling=self.f_ruling,
             )
 
         elif self.surface_shape_type == 1:
@@ -428,16 +429,13 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
                 coating=None,
                 coating_thickness=None,
                 order=self.order,
-                f_ruling=0 if (self.f_ruling == 0) else 5,
+                f_ruling=self.f_ruling,
                 #
-                surface_calculation=SurfaceCalculation.EXTERNAL,
+                # surface_calculation=SurfaceCalculation.EXTERNAL,
+                radius=self.spherical_radius,
                 is_cylinder=self.is_cylinder,
                 cylinder_direction=self.cylinder_orientation, #  Direction:  TANGENTIAL = 0  SAGITTAL = 1
                 convexity=numpy.logical_not(self.surface_curvature).astype(int), #  Convexity: NONE = -1  UPWARD = 0  DOWNWARD = 1
-                radius=self.spherical_radius,
-                p_focus=0.0,
-                q_focus=0.0,
-                grazing_angle=0.0,
             )
         elif self.surface_shape_type == 5:
             grating = S4ToroidGrating(
@@ -451,7 +449,7 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
                 coating=None,
                 coating_thickness=None,
                 order=self.order,
-                f_ruling=0 if (self.f_ruling == 0) else 5,
+                f_ruling=self.f_ruling,
                 #
                 min_radius=self.torus_minor_radius,
                 maj_radius=self.torus_major_radius,
@@ -469,7 +467,7 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
                 coating=None,
                 coating_thickness=None,
                 order=self.order,
-                f_ruling=0 if (self.f_ruling == 0) else 5,
+                f_ruling=self.f_ruling,
                 conic_coefficients=[
                     self.conic_coefficient_0, self.conic_coefficient_1, self.conic_coefficient_2,
                     self.conic_coefficient_3, self.conic_coefficient_4, self.conic_coefficient_5,
@@ -520,7 +518,7 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
                             coating=None,
                             coating_thickness=None,
                             order=self.order,
-                            f_ruling=0 if (self.f_ruling == 0) else 5,
+                            f_ruling=self.f_ruling,
                             )
                         )
         else:
@@ -543,6 +541,19 @@ class OWGrating(OWOpticalElementWithSurfaceShape):
             else: raise NotImplementedError("surface_shape_type not yet implemented!")
 
 
+class OWGrating(_OWGrating):
+    name = "Generic Grating"
+    description = "Shadow Grating"
+    icon = "icons/plane_grating.png"
+
+    priority = 1.390
+
+    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
+    inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "setVlsPgmPreProcessorData"))
+
+    def get_oe_type(self):
+        return "grating", "Grating"
+
 if __name__ == "__main__":
     from shadow4.beamline.s4_beamline import S4Beamline
     from shadow4.sources.source_geometrical.source_geometrical import SourceGeometrical
