@@ -102,7 +102,7 @@ class _OWMirror(OWOpticalElementWithSurfaceShape):
                           "                  beta: ", labelWidth=180, valueType=float,
                           orientation="horizontal", tooltip="refraction_index_beta")
 
-        self.material_refl_box = oasysgui.widgetBox(self.reflectivity_flag_box, "", addSpace=False, orientation="vertical", height=150)
+        self.material_refl_box = oasysgui.widgetBox(self.reflectivity_flag_box, "", addSpace=False, orientation="vertical")
         oasysgui.lineEdit(self.material_refl_box, self, "coating_material",
                           "Coating material (formula): ", labelWidth=180, valueType=str,
                           orientation="horizontal", tooltip="coating_material")
@@ -111,9 +111,11 @@ class _OWMirror(OWOpticalElementWithSurfaceShape):
                           "Coating density [g/cm^3]: ", labelWidth=180, valueType=float,
                           orientation="horizontal", tooltip="coating_density")
 
-        oasysgui.lineEdit(self.material_refl_box, self, "coating_roughness",
-                          "Coating rouughness rms [A]: ", labelWidth=180, valueType=float,
+        self.roughness_refl_box = oasysgui.widgetBox(self.reflectivity_flag_box, "", addSpace=False, orientation="vertical")
+        oasysgui.lineEdit(self.roughness_refl_box, self, "coating_roughness",
+                          "Coating roughness rms [A]: ", labelWidth=180, valueType=float,
                           orientation="horizontal", tooltip="coating_roughness")
+
         self.reflectivity_tab_visibility()
 
     #########################################################
@@ -124,18 +126,23 @@ class _OWMirror(OWOpticalElementWithSurfaceShape):
         self.file_refl_box.setVisible(False)
         self.refraction_index_box.setVisible(False)
         self.material_refl_box.setVisible(False)
+        self.roughness_refl_box.setVisible(False)
 
         if self.reflectivity_flag == 1:
             self.reflectivity_flag_box.setVisible(True)
 
-        if self.reflectivity_source == 1: self.refraction_index_box.setVisible(True)
+        if self.reflectivity_source == 1:
+            self.refraction_index_box.setVisible(True)
 
         if self.reflectivity_source in [0, 2, 3, 4]:
             self.file_refl_box.setVisible(True)
 
-        if self.reflectivity_source in [5,6]:
+        if self.reflectivity_source in [5, 6]:
             self.material_refl_box.setVisible(True)
 
+        if self.reflectivity_source in [0, 1, 5, 6]:
+            self.roughness_refl_box.setVisible(True)
+
     def select_file_refl(self):
         self.le_file_refl.setText(oasysgui.selectFileFromDialog(self, self.file_refl, "Select File with Reflectivity")) #, file_extension_filter="Data Files (*.dat)"))
 
@@ -158,8 +165,8 @@ class _OWMirror(OWOpticalElementWithSurfaceShape):
             self.surface_shape_type = 0
             self.surface_shape_tab_visibility()
 
-            self.source_plane_distance = data.d_source_plane_to_mirror * 1e-3
-            self.image_plane_distance =  data.d_mirror_to_grating/2 * 1e-3
+            self.source_plane_distance = data.d_source_plane_to_mirror
+            self.image_plane_distance =  data.d_mirror_to_grating/2
             self.angles_respect_to = 0
             self.incidence_angle_deg  = (data.alpha + data.beta)/2
             self.reflection_angle_deg = (data.alpha + data.beta)/2
@@ -330,7 +337,7 @@ class _OWMirror(OWOpticalElementWithSurfaceShape):
                 boundary_shape=self.get_boundary_shape(),
                 surface_calculation=self.surface_shape_parameters, # INTERNAL = 0  EXTERNAL = 1
                 min_radius=self.torus_minor_radius,
-                maj_radius=self.torus_major_radius + self.torus_minor_radius, # tangential radius
+                maj_radius=self.torus_major_radius, # tangential radius
                 f_torus=self.toroidal_mirror_pole_location,
                 p_focus=self.get_focusing_p(),
                 q_focus=self.get_focusing_q(),
@@ -419,7 +426,7 @@ class OWMirror(_OWMirror):
     description = "Shadow Mirror"
     icon        = "icons/plane_mirror.png"
 
-    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
+    inputs = copy.deepcopy(_OWMirror.inputs)
     inputs.append(("PreRefl PreProcessor Data", PreReflPreProcessorData, "set_PreReflPreProcessorData"))
     inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "set_VlsPgmPreProcessorData"))
 
@@ -438,21 +445,21 @@ if __name__ == "__main__":
                                      sigma_yp=1.37498e-06)
 
         # Gaussian undulator
-        from shadow4.sources.undulator.s4_undulator import S4Undulator
-        sourceundulator = S4Undulator(
+        from shadow4.sources.undulator.s4_undulator_gaussian import S4UndulatorGaussian
+        sourceundulator = S4UndulatorGaussian(
             period_length=0.0159999,
             number_of_periods=100,
-            emin=2700.136,
-            emax=2700.136,
+            photon_energy=2700.136,
+            delta_e=0.0,
             flag_emittance=1,  # Use emittance (0=No, 1=Yes)
         )
         sourceundulator.set_energy_monochromatic(2700.14)
 
-        from shadow4.sources.undulator.s4_undulator_light_source import S4UndulatorLightSource
-        light_source = S4UndulatorLightSource(name='GaussianUndulator', electron_beam=electron_beam,
+        from shadow4.sources.undulator.s4_undulator_gaussian_light_source import S4UndulatorGaussianLightSource
+        light_source = S4UndulatorGaussianLightSource(name='GaussianUndulator', electron_beam=electron_beam,
                                               magnetic_structure=sourceundulator, nrays=5000, seed=5676561)
 
-        beam = light_source.get_beam_in_gaussian_approximation()
+        beam = light_source.get_beam()
 
         return ShadowData(beam=beam, beamline=S4Beamline(light_source=light_source))
 
