@@ -23,8 +23,13 @@ from syned.widget.widget_decorator import WidgetDecorator
 
 from shadow4.sources.bending_magnet.s4_bending_magnet import S4BendingMagnet
 from shadow4.sources.bending_magnet.s4_bending_magnet_light_source import S4BendingMagnetLightSource
+from shadow4.tools.logger import set_verbose
 
-class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
+from orangecontrib.shadow4.util.shadow4_util import TriggerToolsDecorator
+from oasys.util.oasys_util import TriggerIn
+
+
+class OWBendingMagnet(OWElectronBeam, WidgetDecorator, TriggerToolsDecorator):
 
     name = "Bending Magnet"
     description = "Shadow Source: Bending Magnet"
@@ -38,6 +43,8 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
                 "type":ShadowData,
                 "doc":"",}]
 
+    TriggerToolsDecorator.append_trigger_input_for_sources(inputs)
+    TriggerToolsDecorator.append_trigger_output(outputs)
 
     number_of_rays = Setting(5000)
     seed = Setting(5676561)
@@ -72,7 +79,7 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
         # bm adv settings
         tab_advanced = oasysgui.createTabPage(self.tabs_control_area, "Advanced Setting")
 
-        box_3 = oasysgui.widgetBox(tab_advanced, "Optional parameters for internal calculation", addSpace=True, orientation="vertical")
+        box_3 = oasysgui.widgetBox(tab_advanced, "Advanced settings", addSpace=True, orientation="vertical")
         oasysgui.lineEdit(box_3, self, "ng_e", "Spectrum number of points", tooltip="ng_e", labelWidth=250, valueType=int, orientation="horizontal")
 
         self.add_specific_bm_plots()
@@ -109,8 +116,8 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
             self.bm_tabs.removeTab(size-1-index)
 
         self.bm_tab = [
-            orangegui.createTabPage(self.bm_tabs, "Wiggler Spectrum"),
-            orangegui.createTabPage(self.bm_tabs, "Wiggler Spectral power")
+            orangegui.createTabPage(self.bm_tabs, "BM Spectrum"),
+            orangegui.createTabPage(self.bm_tabs, "BM Spectral power")
         ]
 
         for tab in self.bm_tab:
@@ -155,14 +162,14 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
     def get_lightsource(self):
         # syned electron beam
         electron_beam = self.get_electron_beam()
-        print("\n\n>>>>>> electron_beam info: ", electron_beam.info())
+        print("\n\n***** electron_beam info: ", electron_beam.info())
 
         if self.type_of_properties == 3:
             flag_emittance = 0
         else:
             flag_emittance = 1
 
-        magnetic_radius = S4BendingMagnet.calculate_magnetic_radius(self.magnetic_field, electron_beam.energy())
+        magnetic_radius = numpy.abs(S4BendingMagnet.calculate_magnetic_radius(self.magnetic_field, electron_beam.energy()))
         length = numpy.abs(self.divergence * magnetic_radius)
 
         print(">>> calculated magnetic_radius = S4BendingMagnet.calculate_magnetic_radius(%f, %f) = %f" %\
@@ -170,14 +177,16 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
 
         print(">>> calculated BM length = divergence * magnetic_radius = %f " % length)
 
-        bm = S4BendingMagnet(magnetic_radius,self.magnetic_field,length,
+        bm = S4BendingMagnet(magnetic_radius,
+                             self.magnetic_field,
+                             length,
                              emin=self.emin,  # Photon energy scan from energy (in eV)
                              emax=self.emax,  # Photon energy scan to energy (in eV)
                              ng_e=self.ng_e,  # Photon energy scan number of points
                              flag_emittance=flag_emittance,  # when sampling rays: Use emittance (0=No, 1=Yes)
                              )
 
-        print("\n\n>>>>>> BM info: ", bm.info())
+        print("\n\n***** BM info: ", bm.info())
 
 
         # S4UndulatorLightSource
@@ -187,12 +196,13 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
                                              nrays=self.number_of_rays,
                                              seed=self.seed)
 
-        print("\n\n>>>>>> S4BendingMagnetLightSource info: ", lightsource.info())
+        print("\n\n***** S4BendingMagnetLightSource info: ", lightsource.info())
 
         return lightsource
 
     def run_shadow4(self):
-
+        set_verbose()
+        self.shadow_output.setText("")
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
 
         self._set_plot_quality()
@@ -218,11 +228,11 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
         # run shadow4
 
         t00 = time.time()
-        print(">>>> starting calculation...")
+        print("***** starting calculation...")
         output_beam = light_source.get_beam()
         photon_energy, flux, spectral_power = light_source.calculate_spectrum()
         t11 = time.time() - t00
-        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
+        print("***** time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
 
 
         #
@@ -236,26 +246,31 @@ class OWBendingMagnet(OWElectronBeam, WidgetDecorator):
         self.progressBarFinished()
 
         #
-        # send beam
+        # send beam and trigger
         #
         self.send("Shadow Data", ShadowData(beam=output_beam,
                                            number_of_rays=self.number_of_rays,
                                            beamline=S4Beamline(light_source=light_source)))
-
+        self.send("Trigger", TriggerIn(new_object=True))
 
     def receive_syned_data(self, data):
         if data is not None:
             if isinstance(data, Beamline):
-                if not data.get_light_source() is None:
+                if data.get_light_source() is not None:
+                    light_source = data.get_light_source()
+                    # electron parameters
+                    if light_source.get_electron_beam() is not None:
+                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
+
                     if isinstance(data.get_light_source().get_magnetic_structure(), BendingMagnet):
                         light_source = data.get_light_source()
-                        self.populate_fields_from_syned_electron_beam(light_source.get_electron_beam())
                     else:
-                        raise ValueError("Syned light source not congruent")
+                        self.type_of_properties = 0 # if not ID defined, use electron moments instead of sigmas
+                        self.set_TypeOfProperties()
                 else:
                     raise ValueError("Syned data not correct: light source not present")
             else:
-                raise ValueError("Syned data not correct")
+                raise ValueError("Syned data not correct: it must be Beamline()")
 
 if __name__ == "__main__":
     from PyQt5.QtWidgets import QApplication
