@@ -5,11 +5,13 @@ import numpy
 from orangecontrib.shadow4.widgets.gui.ow_electron_beam import OWElectronBeam
 from orangecontrib.shadow4.widgets.gui.plots import plot_data1D, plot_data2D, plot_data3D
 from orangecontrib.shadow4.util.shadow4_objects import ShadowData
-
+from orangecontrib.shadow4.util.shadow4_util import TriggerToolsDecorator
+from oasys.util.oasys_util import TriggerIn
 from oasys.widgets import gui as oasysgui
 
 from oasys.util.oasys_util import EmittingStream
 
+
 from syned.beamline.beamline import Beamline
 from syned.storage_ring.magnetic_structures.insertion_device import InsertionDevice
 from syned.widget.widget_decorator import WidgetDecorator
@@ -22,8 +24,11 @@ from shadow4.sources.undulator.s4_undulator import S4Undulator
 from shadow4.sources.undulator.s4_undulator_light_source import S4UndulatorLightSource
 
 from shadow4.beamline.s4_beamline import S4Beamline
+from shadow4.tools.logger import set_verbose
 
-class OWUndulator(OWElectronBeam, WidgetDecorator):
+
+
+class OWUndulator(OWElectronBeam, WidgetDecorator, TriggerToolsDecorator):
 
     name = "Undulator Light Source"
     description = "Undulator Light Source"
@@ -35,7 +40,10 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
 
     outputs = [{"name":"Shadow Data",
                 "type":ShadowData,
-                "doc":"",}]
+                "doc":""}]
+
+    TriggerToolsDecorator.append_trigger_input_for_sources(inputs)
+    TriggerToolsDecorator.append_trigger_output(outputs)
 
     # undulator parameters
     K_vertical = Setting(0.25)  # syned Undulator parameter
@@ -84,7 +92,7 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
 
 
     def __init__(self):
-        super().__init__()
+        super().__init__(show_energy_spread=True)
 
         tab_undulator = oasysgui.createTabPage(self.tabs_control_area, "Undulator Setting")
 
@@ -145,7 +153,7 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
 
         # code far field
         left_box_11 = oasysgui.widgetBox(tab_advanced, "Far field simulation", addSpace=False, orientation="vertical")
-        orangegui.comboBox(left_box_11, self, "code_undul_phot", label="Code (for far fielf and backpropagation)", tooltip="code_undul_phot",
+        orangegui.comboBox(left_box_11, self, "code_undul_phot", label="Code (for far field and backpropagation)", tooltip="code_undul_phot",
                            items=["internal", "pysru+wofry", "srw"], labelWidth=260, orientation="horizontal",
                            callback=self.set_visibility)
         oasysgui.lineEdit(left_box_11, self, "distance", "Distance to far field plane [m]", tooltip="distance",
@@ -255,7 +263,7 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
     def get_lightsource(self):
         # syned
         electron_beam = self.get_electron_beam()
-        print("\n\n>>>>>> ElectronBeam info: ", electron_beam.info(), type(electron_beam))
+        print("\n\n***** ElectronBeam info: ", electron_beam.info(), type(electron_beam))
 
         if self.type_of_properties == 3:
             flag_emittance = 0
@@ -287,9 +295,9 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
             flag_backprop_recalculate_source=self.flag_backprop_recalculate_source,
             flag_backprop_weight=self.flag_backprop_weight,
             weight_ratio=self.weight_ratio,
+            flag_energy_spread=self.flag_energy_spread,
             )
 
-
         # S4undulatorLightSource
         lightsource = S4UndulatorLightSource(name='undulator',
                                            electron_beam=electron_beam,
@@ -303,12 +311,13 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
             else:                     lightsource.set_energy_at_resonance(harmonic_number=self.harmonic, delta_e=self.delta_e)
 
 
-        print("\n\n>>>>>> S4undulatorLightSource info: ", lightsource.info())
+        print("\n\n***** S4undulatorLightSource info: ", lightsource.info())
 
         return lightsource
 
     def run_shadow4(self):
-
+        set_verbose()
+        self.shadow_output.setText("")
         self.lightsource = None # clean
 
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
@@ -335,10 +344,10 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
         # run shadow4
         #
         t00 = time.time()
-        print(">>>> starting calculation...")
+        print("***** starting calculation...")
         output_beam = light_source.get_beam()
         t11 = time.time() - t00
-        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
+        print("***** time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
 
         self.lightsource = light_source
 
@@ -353,12 +362,14 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
         self.progressBarFinished()
 
         #
-        # send beam
+        # send beam and trigger
         #
         self.send("Shadow Data", ShadowData(beam=output_beam,
                                            number_of_rays=self.number_of_rays,
                                            beamline=S4Beamline(light_source=light_source)))
 
+        self.send("Trigger", TriggerIn(new_object=True))
+
     def get_title_for_stack_view_flux(self, idx):
         photon_energy = self.lightsource.get_result_dictionary()['photon_energy']
         return "Units: Photons/s/eV/rad2; Photon energy: %8.3f eV"%(photon_energy[idx])
@@ -448,12 +459,12 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
             #
             # power density
             #
-            intens_xy, vx, vz = self.lightsource.get_power_density_interpolated_cartesian()
+            intens_xy, vx, vz = self.lightsource.get_power_density_interpolated_cartesian() # in W/rad2
             if is_monochromatic:
                 title="power density W/mrad2/eV"
             else:
                 title="power density W/mrad2"
-            self.plot_undulator_item2D(4, intens_xy, 1e6 * vx, 1e6 * vz,
+            self.plot_undulator_item2D(4, 1e-6 * intens_xy, 1e6 * vx, 1e6 * vz,
                              title=title, xtitle="vx [urad]", ytitle="vz [rad]")
 
             # spectra
@@ -505,13 +516,14 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
         if data is not None:
             if isinstance(data, Beamline):
-                if not data.get_light_source() is None:
+                if data.get_light_source() is not None:
+                    light_source = data.get_light_source()
+                    # electron parameters
+                    if light_source.get_electron_beam() is not None:
+                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
+
                     if isinstance(data.get_light_source().get_magnetic_structure(), InsertionDevice):
                         print(data.get_light_source().get_magnetic_structure(), InsertionDevice)
-                        light_source = data.get_light_source()
-
-                        # electron parameters
-                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
                         # undulator parameters
                         w = light_source.get_magnetic_structure()
                         self.K_vertical        = w.K_vertical()
@@ -520,14 +532,15 @@ class OWUndulator(OWElectronBeam, WidgetDecorator):
                         #others
                         self.set_at_resonance = 1
                         self.is_monochromatic = 1
-
-                        self.set_visibility()
                     else:
-                        raise ValueError("Syned light source not congruent")
+                        self.type_of_properties = 0 # if not ID defined, use electron moments instead of sigmas
+                        self.set_TypeOfProperties()
+
+                    self.set_visibility()
                 else:
                     raise ValueError("Syned data not correct: light source not present")
             else:
-                raise ValueError("Syned data not correct")
+                raise ValueError("Syned data not correct: it must be Beamline()")
 
 
 if __name__ == "__main__":
