@@ -24,8 +24,13 @@ from syned.widget.widget_decorator import WidgetDecorator
 
 from shadow4.sources.undulator.s4_undulator_gaussian import S4UndulatorGaussian
 from shadow4.sources.undulator.s4_undulator_gaussian_light_source import S4UndulatorGaussianLightSource
+from shadow4.tools.logger import set_verbose
 
-class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
+from orangecontrib.shadow4.util.shadow4_util import TriggerToolsDecorator
+from oasys.util.oasys_util import TriggerIn
+
+
+class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator, TriggerToolsDecorator):
 
     name = "Undulator Gaussian"
     description = "Shadow Source: Undulator Gaussian"
@@ -39,9 +44,12 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
                 "type":ShadowData,
                 "doc":"",}]
 
+    TriggerToolsDecorator.append_trigger_input_for_sources(inputs)
+    TriggerToolsDecorator.append_trigger_output(outputs)
+
     undulator_length = Setting(4.0)
     energy = Setting(15000.0)
-    delta_e = Setting(1500.0)
+    delta_e = Setting(0)
     number_of_rays = Setting(5000)
     seed = Setting(5676561)
 
@@ -75,7 +83,7 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
 
         #
         box_2 = oasysgui.widgetBox(tab_bas, "Sampling rays", addSpace=True, orientation="vertical")
-        oasysgui.lineEdit(box_2, self, "energy", "Set resonance at energy [eV]", tooltip="energy", labelWidth=250, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(box_2, self, "energy", "Set photon energy [eV]", tooltip="energy", labelWidth=250, valueType=float, orientation="horizontal")
         oasysgui.lineEdit(box_2, self, "delta_e", "Delta Energy [eV]", tooltip="delta_e", labelWidth=250, valueType=float, orientation="horizontal")
         oasysgui.lineEdit(box_2, self, "number_of_rays", "Number of Rays", tooltip="number_of_rays", labelWidth=250, valueType=int, orientation="horizontal")
         oasysgui.lineEdit(box_2, self, "seed", "Seed", tooltip="seed", labelWidth=250, valueType=int, orientation="horizontal")
@@ -249,11 +257,6 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
             flux_central_cone=float(self.flux_central_cone),
         )
 
-        if self.delta_e == 0:
-            sourceundulator.set_energy_monochromatic(self.energy)
-        else:
-            sourceundulator.set_energy_box(self.energy-0.5*self.delta_e, self.energy+0.5*self.delta_e,)
-
         # S4UndulatorLightSource
         lightsource = S4UndulatorGaussianLightSource(name='GaussianUndulator',
                                              electron_beam=electron_beam,
@@ -261,12 +264,13 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
                                              nrays=self.number_of_rays,
                                              seed=self.seed)
 
-        print("\n\n>>>>>> S4UndulatorLightSource info: ", lightsource.info())
+        print("\n\n***** S4UndulatorLightSource info: ", lightsource.info())
 
         return lightsource
 
     def run_shadow4(self):
-
+        set_verbose()
+        self.shadow_output.setText("")
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
 
         self._set_plot_quality()
@@ -287,10 +291,10 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
 
         # run shadow4
         t00 = time.time()
-        print(">>>> starting calculation...")
+        print("***** starting calculation...")
         output_beam = light_source.get_beam()
         t11 = time.time() - t00
-        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
+        print("***** time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
 
         self.lightsource = light_source
 
@@ -304,35 +308,39 @@ class OWUndulatorGaussian(OWElectronBeam, WidgetDecorator):
         self.progressBarFinished()
 
         #
-        # send beam
+        # send beam and trigger
         #
         self.send("Shadow Data", ShadowData(beam=output_beam,
                                            number_of_rays=self.number_of_rays,
                                            beamline=S4Beamline(light_source=light_source)))
+        self.send("Trigger", TriggerIn(new_object=True))
 
 
     def receive_syned_data(self, data):
         if data is not None:
             if isinstance(data, Beamline):
-                if not data.get_light_source() is None:
+                if data.get_light_source() is not None:
+                    light_source = data.get_light_source()
+                    # electron parameters
+                    if light_source.get_electron_beam() is not None:
+                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
+
                     if isinstance(data.get_light_source().get_magnetic_structure(), Undulator):
                         light_source = data.get_light_source()
-
                         self.energy =  round(light_source.get_magnetic_structure().resonance_energy(light_source.get_electron_beam().gamma()))
                         self.delta_e = 0.0
                         self.undulator_length = light_source.get_magnetic_structure().length()
                         self.period_length = light_source.get_magnetic_structure().period_length()
                         self.plot_Kmax = light_source.get_magnetic_structure().K_vertical()
-
-
-                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
-
                     else:
-                        raise ValueError("Syned light source not congruent")
+                        self.type_of_properties = 0 # if not ID defined, use electron moments instead of sigmas
+                        self.set_TypeOfProperties()
                 else:
                     raise ValueError("Syned data not correct: light source not present")
+
+
             else:
-                raise ValueError("Syned data not correct")
+                raise ValueError("Syned data not correct: it must be Beamline()")
 
 if __name__ == "__main__":
     from PyQt5.QtWidgets import QApplication
