@@ -1,5 +1,8 @@
 import sys
 import time
+import numpy
+
+from PyQt5.QtWidgets import QMessageBox
 
 from orangecontrib.shadow4.widgets.gui.ow_electron_beam import OWElectronBeam
 from orangecontrib.shadow4.widgets.gui.plots import plot_data1D
@@ -20,8 +23,12 @@ from orangewidget.settings import Setting
 from shadow4.sources.wiggler.s4_wiggler import S4Wiggler
 from shadow4.sources.wiggler.s4_wiggler_light_source import S4WigglerLightSource
 from shadow4.beamline.s4_beamline import S4Beamline
+from shadow4.tools.logger import set_verbose
 
-class OWWiggler(OWElectronBeam, WidgetDecorator):
+from orangecontrib.shadow4.util.shadow4_util import TriggerToolsDecorator
+from oasys.util.oasys_util import TriggerIn
+
+class OWWiggler(OWElectronBeam, WidgetDecorator, TriggerToolsDecorator):
 
     name = "Wiggler Light Source"
     description = "Wiggler Light Source"
@@ -36,6 +43,9 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                 "type":ShadowData,
                 "doc":"",}]
 
+    TriggerToolsDecorator.append_trigger_input_for_sources(inputs)
+    TriggerToolsDecorator.append_trigger_output(outputs)
+
     magnetic_field_source = Setting(1)
     number_of_periods = Setting(1)
     k_value = Setting(10.0)
@@ -59,10 +69,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
     ng_j = Setting(501)
     psi_interval_number_of_points = Setting(101)
 
-    epsi_dx = Setting(0.0)
-    epsi_dz = Setting(0.0)
-
-    flag_interpolation = Setting(2)
+    flag_interpolation = Setting(0)
 
     plot_wiggler_graph = 1
 
@@ -139,12 +146,8 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         oasysgui.lineEdit(left_box_adv, self, "ng_j", "Number of Points in e trajectory (per period)", labelWidth=280, tooltip="ng_j", valueType=int, orientation="horizontal")
         oasysgui.lineEdit(left_box_adv, self, "psi_interval_number_of_points", "Number of Points in sampling vertical angle", labelWidth=280, tooltip="psi_interval_number_of_points", valueType=int, orientation="horizontal")
 
-
-        oasysgui.lineEdit(left_box_adv, self, "epsi_dx", "Distance from waist X [m]", labelWidth=260, tooltip="epsi_dx", valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(left_box_adv, self, "epsi_dz", "Distance from waist Z [m]", labelWidth=260, tooltip="epsi_dz", valueType=float, orientation="horizontal")
-
         orangegui.comboBox(left_box_adv, self, "flag_interpolation", tooltip="flag_interpolation", label="Sample psi via interpolation",
-                           items=["No (accurate, exact Bessel)", "Yes (good for mono or quasi monochromatic)", "No (accurate, approx Bessel)"], labelWidth=260, orientation="horizontal")
+                           items=["No (accurate, exact Bessel)", "Yes (good for mono or quasi monochromatic)", "Yes (ray by ray)"], labelWidth=260, orientation="horizontal")
 
 
         # wiggler plots
@@ -226,35 +229,33 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
             flag_emittance = 0
         else:
             flag_emittance = 1
+
+        if self.e_min == self.e_max:
+            ng_e = 1
+        else:
+            ng_e = self.ng_e
         # S4Wiggler
         if self.magnetic_field_source == 0:
             sourcewiggler = S4Wiggler(
                     magnetic_field_periodic  = 1,   # 0=external, 1=periodic
                     file_with_magnetic_field = "",  # useful if magnetic_field_periodic=0
-                    K_vertical               = self.k_value,
-                    period_length            = self.id_period,
-                    number_of_periods        = self.number_of_periods, # syned Wiggler pars: useful if magnetic_field_periodic=1
-                    emin                     = self.e_min,     # Photon energy scan from energy (in eV)
-                    emax                     = self.e_max,     # Photon energy scan to energy (in eV)
-                    ng_e                     = self.ng_e,      # Photon energy scan number of points
-                    ng_j                     = self.ng_j ,     # Number of points in electron trajectory (per period) for internal calculation only
+                    K_vertical                    = self.k_value,
+                    period_length                 = self.id_period,
+                    number_of_periods             = self.number_of_periods, # syned Wiggler pars: useful if magnetic_field_periodic=1
+                    emin                          = self.e_min,     # Photon energy scan from energy (in eV)
+                    emax                          = self.e_max,     # Photon energy scan to energy (in eV)
+                    ng_e                          = ng_e,      # Photon energy scan number of points
+                    ng_j                          = self.ng_j ,     # Number of points in electron trajectory (per period) for internal calculation only
                     psi_interval_number_of_points = self.psi_interval_number_of_points,
-                    flag_interpolation       = self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
-                    flag_emittance           = flag_emittance, # Use emittance (0=No, 1=Yes)
-                    shift_x_flag             = 0,
-                    shift_x_value            = 0.0,
-                    shift_betax_flag         = 0,
-                    shift_betax_value        = 0.0,
-                    epsi_dx                  = self.epsi_dx,
-                    epsi_dz                  = self.epsi_dz,
+                    flag_interpolation            = self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
+                    flag_emittance                = flag_emittance, # Use emittance (0=No, 1=Yes)
+                    shift_x_flag                  = 0,
+                    shift_x_value                 = 0.0,
+                    shift_betax_flag              = 0,
+                    shift_betax_value             = 0.0,
                     )
 
         elif self.magnetic_field_source == 1:
-            if self.e_min == self.e_max:
-                ng_e = 1
-            else:
-                ng_e = self.ng_e
-
             sourcewiggler = S4Wiggler(
                 magnetic_field_periodic   = 0,
                 file_with_magnetic_field  = self.file_with_b_vs_y,
@@ -262,8 +263,8 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                 emax                      = self.e_max,
                 ng_e                      = ng_e,
                 ng_j                      = self.ng_j,
-                psi_interval_number_of_points=self.psi_interval_number_of_points,
-                flag_interpolation=self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
+                psi_interval_number_of_points = self.psi_interval_number_of_points,
+                flag_interpolation        = self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
                 flag_emittance            = flag_emittance,  # Use emittance (0=No, 1=Yes)
                 shift_x_flag              = 4,
                 shift_x_value             = 0.0,
@@ -287,7 +288,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                         shift_betax_flag=self.shift_betax_flag,
                         shift_betax_value=self.shift_betax_value)
 
-        print(">>>>>> \n\n S4Wiggler get_info: \n", sourcewiggler.get_info())
+        print("***** \n\n S4Wiggler get_info: \n", sourcewiggler.get_info())
 
         # S4WigglerLightSource
         lightsource = S4WigglerLightSource(name='wiggler',
@@ -296,12 +297,14 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                                            nrays=self.number_of_rays,
                                            seed=self.seed)
 
-        print("\n\n>>>>>> S4WigglerLightSource info: \n", lightsource.info())
+        print("\n\n***** S4WigglerLightSource info: \n", lightsource.info())
 
         return lightsource
 
 
     def run_shadow4(self):
+        set_verbose()
+        self.shadow_output.setText("")
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
 
         self._set_plot_quality()
@@ -328,11 +331,11 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
 
         self.progressBarSet(10)
         t00 = time.time()
-        print(">>>> starting shadow calculation...")
+        print("***** starting shadow calculation...")
         output_beam = light_source.get_beam()
         photon_energy, flux, spectral_power = light_source.calculate_spectrum()
         t11 = time.time() - t00
-        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
+        print("***** time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
 
         #
         # plots
@@ -343,11 +346,12 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         self.progressBarFinished()
 
         #
-        # send beam
+        # send beam and trigger
         #
         self.send("Shadow Data", ShadowData(beam=output_beam,
                                            number_of_rays=self.number_of_rays,
                                            beamline=S4Beamline(light_source=light_source)))
+        self.send("Trigger", TriggerIn(new_object=True))
 
     def refresh_specific_wiggler_plots(self, lightsource=None, e=None, f=None, w=None):
 
@@ -366,7 +370,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                                   title="Magnetic Field",xtitle="y [m]",ytitle="B [T]")
 
             self.plot_widget_item(traj[1, :],traj[6, :],1,
-                                  title="Electron curvature",xtitle="y [m]",ytitle="cirvature [m^-1]")
+                                  title="Electron curvature",xtitle="y [m]",ytitle="curvature [m^-1]")
 
             self.plot_widget_item(traj[1, :],traj[3, :],2,
                                   title="Electron velocity",xtitle="y [m]",ytitle="BetaX")
@@ -392,7 +396,12 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         sys.stdout = EmittingStream(textWritten=self._write_stdout)
         if data is not None:
             if isinstance(data, Beamline):
-                if not data.get_light_source() is None:
+                if data.get_light_source() is not None:
+                    light_source = data.get_light_source()
+                    # electron parameters
+                    if light_source.get_electron_beam() is not None:
+                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
+
                     if isinstance(data.get_light_source().get_magnetic_structure(), InsertionDevice):
                         print(data.get_light_source().get_magnetic_structure(), InsertionDevice)
                         light_source = data.get_light_source()
@@ -404,14 +413,13 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                         self.number_of_periods = int(w.number_of_periods())
                         self.id_period = w.period_length()
                         self.k_value = w.K_vertical()
-
-                        self.populate_fields_from_electron_beam(light_source.get_electron_beam())
                     else:
-                        raise ValueError("Syned light source not congruent")
+                        self.type_of_properties = 0 # if not ID defined, use electron moments instead of sigmas
+                        self.set_TypeOfProperties()
                 else:
                     raise ValueError("Syned data not correct: light source not present")
             else:
-                raise ValueError("Syned data not correct")
+                raise ValueError("Syned data not correct: it must be Beamline()")
 
 if __name__ == "__main__":
     from PyQt5.QtWidgets import QApplication
